import os
import requests
from requests_toolbelt.multipart.encoder import MultipartEncoder, MultipartEncoderMonitor

def check_custom_name(check_url: str, custom_name: str) -> bool:
	"""
	Verifica si un nombre personalizado ya existe en el servidor.

	:param check_url: URL del endpoint para verificar el nombre.
	:param custom_name: Nombre personalizado a verificar.
	:return: True si el nombre ya existe, False en caso contrario.
	:raises ValueError: Si la URL o el nombre están vacíos.
	:raises requests.RequestException: Si hay un error en la solicitud HTTP.
	"""
	if not check_url or not custom_name:
		raise ValueError("La URL de verificación y el nombre personalizado son obligatorios.")

	try:
		response = requests.get(f"{check_url}?name={custom_name}", timeout=10)
		response.raise_for_status()  # Lanza una excepción si el estado no es 200
		data = response.json()
		return data.get("exists", False)
	except requests.exceptions.RequestException as e:
		raise requests.RequestException(f"Error en la solicitud HTTP: {e}")
	except ValueError:
		raise ValueError("Error al procesar la respuesta del servidor. Asegúrate de que devuelve JSON válido.")


def upload_file(file_path: str, upload_url: str, custom_name: str = None, expire: int = 24, progress_callback=None) -> dict:
	"""
	Sube un archivo al servidor utilizando una solicitud HTTP POST con multipart/form-data.

	:param file_path: Ruta del archivo a subir.
	:param upload_url: URL del endpoint de subida.
	:param custom_name: (Opcional) Nombre personalizado para el archivo.
	:param expire: (Opcional) Tiempo de expiración en horas (por defecto 24).
	:param progress_callback: (Opcional) Función callback que recibe el progreso (0-100).
	:return: Diccionario con la respuesta JSON del servidor.
	:raises ValueError: Si algún parámetro es inválido.
	:raises requests.RequestException: Si hay un error en la subida del archivo.
	"""
	if not os.path.isfile(file_path):
		raise ValueError("El archivo especificado no existe o no es un archivo válido.")

	if expire <= 0:
		raise ValueError("El tiempo de expiración debe ser mayor a 0.")

	file_size = os.path.getsize(file_path)
	max_size = 1024 * 1024 * 1024  # 1GB en bytes

	if file_size > max_size:
		raise ValueError("El archivo es demasiado grande. El límite es 1GB.")

	try:
		with open(file_path, 'rb') as f:
			# Preparar los campos del formulario
			fields = {
				"file": (os.path.basename(file_path), f, "application/octet-stream"),
				"expire": str(expire)
			}
			if custom_name:
				fields["custom_path"] = custom_name

			# Configurar el encoder para el monitoreo del progreso
			encoder = MultipartEncoder(fields=fields)
			total_size = encoder.len  # Tamaño total del archivo

			def _callback(monitor):
				"""Función para actualizar el progreso de la subida."""
				if progress_callback:
					progress = int((monitor.bytes_read / total_size) * 100)
					progress_callback(progress)

			monitor = MultipartEncoderMonitor(encoder, _callback)
			headers = {"Content-Type": monitor.content_type}

			# Enviar la solicitud HTTP POST
			response = requests.post(upload_url, data=monitor, headers=headers, timeout=60)
			response.raise_for_status()  # Lanza una excepción si la respuesta no es 200

			return response.json()
	except requests.exceptions.RequestException as e:
		raise requests.RequestException(f"Error en la subida del archivo: {e}")
	except ValueError as e:
		raise ValueError(f"Error en los datos proporcionados: {e}")
